#ifndef _BRAIN_HPP_
#define _BRAIN_HPP_

#include <vector>
using std::vector;
#include <string>
using std::string;

enum InstructionType {
  InstructionType_Sense,
  InstructionType_Mark,
  InstructionType_Unmark,
  InstructionType_PickUp,
  InstructionType_Drop,
  InstructionType_Turn,
  InstructionType_Move,
  InstructionType_Flip
};

enum SenseDir {
  SenseDir_Here,
  SenseDir_Ahead,
  SenseDir_LeftAhead,
  SenseDir_RightAhead
};

enum Cond {
  Cond_Friend,
  Cond_Foe,
  Cond_FriendWithFood,
  Cond_FoeWithFood,
  Cond_Food,
  Cond_Rock,
  Cond_Marker,
  Cond_FoeMarker,
  Cond_Home,
  Cond_FoeHome
};

enum LR {
  LR_Left,
  LR_Right
};

class Instruction
{
public:
  Instruction();
  ~Instruction();

  InstructionType getInstructionType() const { return m_instructionType; }
  void setInstructionType(InstructionType instructionType)
  { m_instructionType = instructionType; }

  int getNumArgs() const;
  void setNumArgs(int numArgs);

  int getArg(int argNum) const;
  void setArg(int argNum, int arg);

private:
  InstructionType m_instructionType;
  vector<int> m_args;
};

class Brain
{
public:
  Brain();
  ~Brain();

  int getNumInstructions() const;
  const Instruction &getInstruction(int instructionNum) const;
  void addInstruction(const Instruction &instruction);

private:
  vector<Instruction> m_instructions;
};

bool readBrain(const string &fileName, Brain *pBrain);

bool formatBrain(const Brain &brain, string *pStr);

#endif
