#ifndef _WORLD_HPP_
#define _WORLD_HPP_

#include <vector>
using std::vector;
#include <string>
using std::string;

#include "Color.hpp"

enum CellType {
  CellType_Rocky,
  CellType_Clear,
  CellType_RedAnthill,
  CellType_BlackAnthill
};

class Cell
{
public:
  Cell();
  ~Cell();

  CellType getCellType() const { return m_cellType; }
  void setCellType(CellType cellType) { m_cellType = cellType; }

  int getAntId() const { return m_antId; }
  void setAntId(int antId) { m_antId = antId; }

  int getFood() const { return m_food; }
  void setFood(int food) { m_food = food; }

  bool getMarker(Color color, int marker) const;
  void setMarker(Color color, int marker, bool value);
  bool getAnyMarker(Color color) const;

private:
  CellType m_cellType;
  int m_antId; // -1 if no ant
  int m_food;
  bool m_markers[Color_Count][6];
};

class Ant
{
public:
  Ant();
  ~Ant();

  Color getColor() const { return m_color; }
  void setColor(Color color) { m_color = color; }

  int getState() const { return m_state; }
  void setState(int state) { m_state = state; }

  int getResting() const { return m_resting; }
  void setResting(int resting) { m_resting = resting; }

  int getDir() const { return m_dir; }
  void setDir(int dir) { m_dir = dir; }

  bool getHasFood() const { return m_hasFood; }
  void setHasFood(bool hasFood) { m_hasFood = hasFood; }

  int getX() const { return m_x; }
  void setX(int x) { m_x = x; }

  int getY() const { return m_y; }
  void setY(int y) { m_y = y; }

private:
  Color m_color;
  int m_state;
  int m_resting;
  int m_dir;
  bool m_hasFood;
  int m_x;
  int m_y;
};

class World
{
public:
  World();
  ~World();

  int getWidth() const { return m_width; }
  int getHeight() const { return m_height; }
  void setSize(int width, int height); // clobbers the data

  int getNumAnts() const;
  void setNumAnts(int numAnts);

  const Cell &cell(int x, int y) const;
  Cell &cell(int x, int y);

  const Ant &ant(int antId) const;
  Ant &ant(int antId);

  void setSeed(unsigned int seed);
  int randomInt(int n);

  int getScore(Color color) const;
  void setScore(Color color, int score);
  void adjustScore(CellType cellType, int delta);

private:
  vector<Cell> m_cells;
  vector<Ant> m_ants;
  int m_width;
  int m_height;
  unsigned int m_seed;
  int m_scores[Color_Count];
};

bool readWorld(const string &fileName, World *pWorld);
void printWorld(const World &world);
void printWorldAnsi(const World &world);
void printWorldInOfficialFormat(const World &world);
void dumpWorld(const World &world);

#endif
