> {-# OPTIONS -Wall -fno-warn-unused-matches #-}

> module AntAsmDirection(parametrizeOnDirection) where

> import ParsecPos
> import AntAsmInstrs

Direction is one of [0..5]; zero (eastbound) is where the ants start,
and the numbers increase clockwise

> type Direction = Int

Generate six copies of the state machine, parametrizing on direction

> parametrizeOnDirection :: [(SourcePos, ParamByDirInstr)] -> [Instr]
> parametrizeOnDirection instrs
>     | numInstrs > 1666 =
>         error ("ERROR: more than 833 instructions not supported")
>     | otherwise = concatMap (\dir -> map (parametrize dir) instrs) [0..5]
>     where numInstrs = length instrs
>           parametrize dir = snd . updateTurn numInstrs dir . project dir

Change everything to reflect instantiation of overloading

> project :: Direction -> (SourcePos, ParamByDirInstr) -> (SourcePos, Instr)
> project currentDir (pos, instr) = (pos, newInstr)
>     where newInstr = mapStateTurn $ mapTurnDir sel $ mapSenseDir sel $
>                      mapCond (mapCondMarker sel . sel) $ mapMarker sel $
>                      mapChance sel $ instr
>           sel = selectDir pos currentDir
>           mapStateTurn i@(Turn turnDir _) =
>               mapState (selectDir pos newDir) i
>                   where newDir = (currentDir + valOfDir turnDir) `mod` 6
>           mapStateTurn i =
>               mapState (selectDir pos currentDir) i

Project an item based on the a given direction

> selectDir :: SourcePos -> Direction -> ParamByDir item -> item
> selectDir pos selDir [item] = item -- not parametrized
> selectDir pos selDir items@[_, _, _, _, _, _]
>     | selDir >= 0 && selDir < 6 = items !! selDir
> selectDir pos selDir items =
>     error (show pos ++ ": invalid index selection: " ++ show selDir)


Change the Turn command's addresses to reflect the current state plus turn

> updateTurn :: Int -> Direction -> (SourcePos, Instr) -> (SourcePos, Instr)
> updateTurn numInstrs currentDir (pos, instr) =
>     let offset = case instr of
>                  Turn dir _ -> valOfDir dir
>                  _          -> 0
>         newDir = (currentDir + offset) `mod` 6
>         updNextState nextState = newDir * numInstrs + nextState
>     in  (pos, mapState updNextState instr)

Values of left and right turns, modulo six

> valOfDir :: TurnDir -> Direction
> valOfDir AntAsmInstrs.Left = 5
> valOfDir AntAsmInstrs.Right = 1
