> {-# OPTIONS -Wall -fno-warn-unused-matches #-}

> module AntAsmFood(parametrizeOnFood) where

> import ParsecPos
> import AntAsmInstrs

Whether this ant is carrying food

> type HasFood = Bool

Generate two copies of the state machine, parametrizing on direction

> parametrizeOnFood :: [(SourcePos, ParamByDirFoodInstr)]
>                   -> [(SourcePos, ParamByDirInstr)]
> parametrizeOnFood instrs
>     | numInstrs > 833 =
>         error ("ERROR: more than 833 instructions not supported")
>     | otherwise = concatMap (\food -> map (parametrize food) instrs)
>                   [False,True]
>     where numInstrs = length instrs
>           parametrize food = updatePickupDrop numInstrs food . project food

Change everything to reflect instantiation of overloading

> project :: HasFood -> (SourcePos, ParamByDirFoodInstr)
>         -> (SourcePos, ParamByDirInstr)
> project hasFood (pos, instr) = (pos, newInstr)
>     where newInstr = mapStateFood $ mapTurnDir sel $ mapSenseDir sel $
>                      mapCond (map (mapCondMarker sel) . sel) $
>                      mapMarker sel $ mapChance sel $ instr
>           sel = map (selectFood pos hasFood)
>           mapStateFood i@(PickUp okStatess badStatess) =
>               let okStatess' = map (selectFood pos True) okStatess
>                   badStatess' = map (selectFood pos hasFood) badStatess
>               in  PickUp okStatess' badStatess'
>           mapStateFood i = mapState (map (selectFood pos hasFood)) i

Project out of item list according to whether we're carrying food

> selectFood :: SourcePos -> HasFood -> ParamByFood item -> item
> selectFood pos hasFood [item] = item -- not parametrized
> selectFood pos False [item, _] = item
> selectFood pos True [_, item] = item
> selectFood pos hasFood items =
>     error (show pos ++ ": invalid food selection: length " ++
>            show (length items))

Change the PickUp and Drop commands to reflect successfully picked up
and dropped food

> updatePickupDrop :: Int -> HasFood -> (SourcePos, ParamByDirInstr)
>                  -> (SourcePos, ParamByDirInstr)
> updatePickupDrop numInstrs True (pos, PickUp okStates badStates) =
>     (pos, PickUp [0] [numInstrs + nextState | nextState <- badStates])
> updatePickupDrop numInstrs False (pos, PickUp okStates badStates) =
>     (pos, PickUp [numInstrs + nextState | nextState <- okStates] badStates)
> updatePickupDrop numInstrs hasFood (pos, Drop nextStates) =
>     (pos, Drop nextStates)
> updatePickupDrop numInstrs True (pos, instr) =
>     (pos, mapState (map (numInstrs +)) instr)
> updatePickupDrop numInstrs False (pos, instr) = (pos, instr)
