> {-# OPTIONS -Wall -fno-warn-unused-matches #-}

> module AntAsmInstrs where

> import ParsecPos

Source program: list of positions and statements

> type SourceProgram = [(SourcePos, SourceStmt)]

Statement: label or instruction (parser result)

> data SourceStmt = Label String | Instr SourceInstr

Ant state machine instructions
Polymorphic in state -- could be label or number

> data InstrT state turndir sensedir condition marker chance
>     = Sense sensedir state state condition
>     | Mark marker state
>     | Unmark marker state
>     | PickUp state state
>     | Drop state
>     | Turn turndir state
>     | Move state state
>     | Flip chance state state
>    deriving (Show)

Final instruction type

> type Instr = InstrT State TurnDir SenseDir (Condition Marker) Marker Chance

Parametrization types: we can parametrize by the direction we face and
whether we're carrying food or not

> type ParamByDir contents = [contents]
> type ParamByFood contents = [contents]
> type ParamByDirFood contents = ParamByDir (ParamByFood contents)

Source instruction type; outer tuple (direction) is either one or six;
inner tuple (foodp) is either one or two

> type SourceInstr = InstrT
>     (ParamByDirFood StateRef)
>     (ParamByDirFood TurnDir)
>     (ParamByDirFood SenseDir)
>     (ParamByDirFood (Condition (ParamByDirFood Marker)))
>     (ParamByDirFood Marker)
>     (ParamByDirFood Chance)

Intermediate instruction type; outer tuple (direction) is either one
or six; inner tuple (foodp) is either one or two

> type ParamByDirFoodInstr = InstrT
>     (ParamByDirFood State)
>     (ParamByDirFood TurnDir)
>     (ParamByDirFood SenseDir)
>     (ParamByDirFood (Condition (ParamByDirFood Marker)))
>     (ParamByDirFood Marker)
>     (ParamByDirFood Chance)

Intermediate instruction type; tuple (direction) is either one or six

> type ParamByDirInstr = InstrT
>     (ParamByDir State)
>     (ParamByDir TurnDir)
>     (ParamByDir SenseDir)
>     (ParamByDir (Condition (ParamByDir Marker)))
>     (ParamByDir Marker)
>     (ParamByDir Chance)

Address of a given state

> type State = Int

Absolute state address or label, used in parsing

> data StateRef = Next | AfterNext | Absolute State | LabelRef String

Direction in which an ant can turn: left or right

> data TurnDir = Left | Right deriving (Enum, Bounded, Show)

Direction in which an ant can sense, relative to current position/bearing

> data SenseDir = Here | Ahead | LeftAhead | RightAhead
>                 deriving (Enum, Bounded, Show)

Markers which ants can leave about; change to datatype with names?

> type Marker = Int

Condition which an ant can sense

> data Condition marker =
>    Friend
>  | Foe
>  | FriendWithFood
>  | FoeWithFood
>  | Food
>  | Rock
>  | Marker marker
>  | FoeMarker
>  | Home
>  | FoeHome

The probability (actually, its inverse) of choosing some path

> type Chance = Int

Pretty-printing for conditions (derived instance is almost right, but
puts parens around Marker ...)

> instance (Show marker) => Show (Condition marker) where
>     show Friend = "Friend"
>     show Foe = "Foe"
>     show FriendWithFood = "FriendWithFood"
>     show FoeWithFood = "FoeWithFood"
>     show Food = "Food"
>     show Rock = "Rock"
>     show (Marker m) = "Marker " ++ show m
>     show FoeMarker = "FoeMarker"
>     show Home = "Home"
>     show FoeHome = "FoeHome"

Replace any state within an instruction

> mapState :: (state1 -> state2)
>          -> InstrT state1 turndir sensedir cond marker chance
>          -> InstrT state2 turndir sensedir cond marker chance
> mapState f (Sense dir st1 st2 cond) = Sense dir (f st1) (f st2) cond
> mapState f (Mark marker st1) = Mark marker (f st1)
> mapState f (Unmark marker st1) = Unmark marker (f st1)
> mapState f (PickUp st1 st2) = PickUp (f st1) (f st2)
> mapState f (Drop st1) = Drop (f st1)
> mapState f (Turn dir st1) = Turn dir (f st1)
> mapState f (Move st1 st2) = Move (f st1) (f st2)
> mapState f (Flip chance st1 st2) = Flip chance (f st1) (f st2)

Replace any turn direction within an instruction

> mapTurnDir :: (turndir1 -> turndir2)
>            -> InstrT state turndir1 sensedir condition marker chance
>            -> InstrT state turndir2 sensedir condition marker chance
> mapTurnDir f (Turn dir st1) = Turn (f dir) st1
> mapTurnDir f (Sense dir st1 st2 cond) = Sense dir st1 st2 cond
> mapTurnDir f (Mark marker st1) = Mark marker st1
> mapTurnDir f (Unmark marker st1) = Unmark marker st1
> mapTurnDir f (PickUp st1 st2) = PickUp st1 st2
> mapTurnDir f (Drop st1) = Drop st1
> mapTurnDir f (Move st1 st2) = Move st1 st2
> mapTurnDir f (Flip chance st1 st2) = Flip chance st1 st2

Replace any sense direction within an instruction

> mapSenseDir :: (sensedir1 -> sensedir2)
>             -> InstrT state turndir sensedir1 condition marker chance
>             -> InstrT state turndir sensedir2 condition marker chance
> mapSenseDir f (Sense dir st1 st2 cond) = Sense (f dir) st1 st2 cond
> mapSenseDir f (Mark marker st1) = Mark marker st1
> mapSenseDir f (Unmark marker st1) = Unmark marker st1
> mapSenseDir f (PickUp st1 st2) = PickUp st1 st2
> mapSenseDir f (Drop st1) = Drop st1
> mapSenseDir f (Turn dir st1) = Turn dir st1
> mapSenseDir f (Move st1 st2) = Move st1 st2
> mapSenseDir f (Flip chance st1 st2) = Flip chance st1 st2

Replace any condition within an instruction

> mapCond :: (cond1 -> cond2)
>         -> InstrT state turndir sensedir cond1 marker chance
>         -> InstrT state turndir sensedir cond2 marker chance
> mapCond f (Sense dir st1 st2 cond) = Sense dir st1 st2 (f cond)
> mapCond f (Mark marker st1) = Mark marker st1
> mapCond f (Unmark marker st1) = Unmark marker st1
> mapCond f (PickUp st1 st2) = PickUp st1 st2
> mapCond f (Drop st1) = Drop st1
> mapCond f (Turn dir st1) = Turn dir st1
> mapCond f (Move st1 st2) = Move st1 st2
> mapCond f (Flip ch st1 st2) = Flip ch st1 st2

Replace any marker within an instruction

> mapMarker :: (marker1 -> marker2)
>           -> InstrT state turndir sensedir condition marker1 chance
>           -> InstrT state turndir sensedir condition marker2 chance
> mapMarker f (Mark marker st) = Mark (f marker) st
> mapMarker f (Unmark marker st) = Unmark (f marker) st
> mapMarker f (Sense dir st1 st2 cond) = Sense dir st1 st2 cond
> mapMarker f (PickUp st1 st2) = PickUp st1 st2
> mapMarker f (Drop st1) = Drop st1
> mapMarker f (Turn dir st1) = Turn dir st1
> mapMarker f (Move st1 st2) = Move st1 st2
> mapMarker f (Flip chance st1 st2) = Flip chance st1 st2

Replace any marker within a condition

> mapCondMarker :: (marker1 -> marker2) -> (Condition marker1)
>               -> (Condition marker2)
> mapCondMarker f (Marker marker) = Marker (f marker)
> mapCondMarker f Friend = Friend
> mapCondMarker f Foe = Foe
> mapCondMarker f FriendWithFood = FriendWithFood
> mapCondMarker f FoeWithFood = FoeWithFood
> mapCondMarker f Food = Food
> mapCondMarker f Rock = Rock
> mapCondMarker f FoeMarker = FoeMarker
> mapCondMarker f Home = Home
> mapCondMarker f FoeHome = FoeHome

Replace any condition coefficient within an instruction

> mapChance :: (chance1 -> chance2)
>           -> InstrT state turndir sensedir condition marker chance1
>           -> InstrT state turndir sensedir condition marker chance2
> mapChance f (Flip chance st1 st2) = Flip (f chance) st1 st2
> mapChance f (Sense dir st1 st2 cond) = Sense dir st1 st2 cond
> mapChance f (Mark marker st1) = Mark marker st1
> mapChance f (Unmark marker st1) = Unmark marker st1
> mapChance f (PickUp st1 st2) = PickUp st1 st2
> mapChance f (Drop st1) = Drop st1
> mapChance f (Turn dir st1) = Turn dir st1
> mapChance f (Move st1 st2) = Move st1 st2
