> {-# OPTIONS -Wall -fno-warn-unused-matches #-}

Resolve label addresses

> module AntAsmResolve (resolveLabels) where

> import ParsecPos
> import AntAsmInstrs
> import FiniteMap

Symbol (label) table

> type SymTab = FiniteMap String (SourcePos, State)

> resolveLabels :: [(SourcePos, SourceStmt)] ->
>                  [(SourcePos, ParamByDirFoodInstr)]
> resolveLabels stmts =
>     let (numberedInstrs, symtab, maxState) = numberStates stmts
>         resolvedInstrs = [resolveInstr symtab maxState st_pos_instr
>                           | st_pos_instr <- numberedInstrs]
>     in  resolvedInstrs

> numberStates :: [(SourcePos, SourceStmt)]
>              -> ([(State, SourcePos, SourceInstr)], SymTab, State)
> numberStates stmts = numberStates' emptyFM 0 [] stmts

> numberStates' :: SymTab -> State -> [(State, SourcePos, SourceInstr)]
>                         -> [(SourcePos, SourceStmt)]
>                         -> ([(State, SourcePos, SourceInstr)], SymTab, State)
> numberStates' symtab nextState revInstrs [] =
>     (reverse revInstrs, symtab, nextState - 1)
> numberStates' symtab nextState revInstrs ((pos, Label name):restStmts) =
>     case lookupFM symtab name of
>     Just (prevPos, _ {- prevState -}) ->
>         error (show pos ++ ": label " ++ show name ++
>                " previously declared at " ++ show prevPos)
>     Nothing -> let symtab' = addToFM symtab name (pos, nextState)
>                in  numberStates' symtab' nextState revInstrs restStmts
> numberStates' symtab nextState revInstrs ((pos, Instr instr):restStmts) =
>     numberStates' symtab (nextState + 1) ((nextState, pos, instr):revInstrs)
>                 restStmts

> resolveInstr :: SymTab -> State -> (State, SourcePos, SourceInstr)
>              -> (SourcePos, ParamByDirFoodInstr)
> resolveInstr symtab maxState (currentState, pos, instr) =
>     (pos, mapState (resolveStates symtab maxState currentState pos) instr)

> resolveStates :: SymTab -> State -> State -> SourcePos
>               -> ParamByDirFood StateRef -> ParamByDirFood State
> resolveStates symtab maxState currentState pos statess =
>     [[resolveState symtab maxState currentState pos state | state <- states]
>      | states <- statess]

> resolveState :: SymTab -> State -> State -> SourcePos -> StateRef -> State
> resolveState symtab maxState currentState pos (Absolute state)
>     | state <= maxState = state
>     | otherwise = error (show pos ++ ": state " ++ show state ++
>                          " lies beyond the end of the program")
> resolveState symtab maxState currentState pos (Next)
>     | currentState < maxState = currentState + 1
>     | otherwise = error (show pos ++ ": next state not defined")
> resolveState symtab maxState currentState pos (AfterNext)
>     | currentState < maxState - 1 = currentState + 2
>     | otherwise = error (show pos ++ ": afternext state not defined")
> resolveState symtab maxState currentState pos (LabelRef name) =
>     case lookupFM symtab name of
>     Nothing -> error (show pos ++ ": state " ++ show name ++ " not defined")
>     Just (_, state) -> state
