> {-# OPTIONS -Wall -fno-warn-unused-matches #-}

Ant assembler

> module Main where

> import System
> import IO
> import Parsec
> import AntAsmParser
> import AntAsmInstrs (SourceProgram)
> import AntAsmResolve
> import AntAsmDirection
> import AntAsmFood

> main :: IO ()
> main = getArgs >>= mainWithArgs

> mainWithArgs :: [String] -> IO ()
> mainWithArgs args | any (`elem` ["-h", "--help", "-help"]) args = usage
> mainWithArgs [] =
>     do source <- getContents
>        let result = parse pProgram "-" source
>        asmParsed result
> mainWithArgs [file] =
>     do result <- parseFromFile pProgram file
>        asmParsed result
> mainWithArgs (_:_:_) =
>     do putStrLn "ERROR: only one source file may be given"
>        exitWith (ExitFailure 1)

> asmParsed :: Either ParseError SourceProgram -> IO ()
> asmParsed (Left err) =
>     do hPutStrLn stderr (show err)
>        exitWith (ExitFailure 1)
> asmParsed (Right program) =
>     do let resolvedInstrs = resolveLabels program
>            foodedInstrs = parametrizeOnFood resolvedInstrs
>            diredInstrs = parametrizeOnDirection foodedInstrs
>        mapM_ print diredInstrs

> usage :: IO ()
> usage =
>     do putStrLn blurb
>        putStrLn "usage:"
>        putStrLn "  antasm source             compile file named source"
>        putStrLn "  antasm                    compile standard input"
>        exitWith ExitSuccess

> blurb :: String
> blurb = "ICFP2004 Ant State Machine Assembler"
