#include <stdio.h>
#include "World.hpp"
#include "Color.hpp"
#include "Brain.hpp"
#include "Sim.hpp"

namespace
{
  bool parseInt(const string &token,
                int *pInt)
  {
    int sz = token.size();
    int result = 0;
    for (int i = 0; i < sz; i++) {
      char ch = token[i];
      if (ch < '0' || ch > '9') {
        return false;
      }
      result *= 10;
      result += (ch - '0');
    }
    *pInt = result;
    return true;
  }
}

int main(int argc, char *argv[])
{
  if (argc != 5) {
    printf("Usage: dump <worldfile> <redbrainfile> <blackbrainfile> <seed>\n");
    return 1;
  }

  string worldFile(argv[1]);
  string brainFiles[2];
  brainFiles[Color_Red] = argv[2];
  brainFiles[Color_Black] = argv[3];

  int seed = 12345;
  if (!parseInt(argv[4], &seed)) {
    printf("Error parsing seed.\n");
    return 1;
  }

  World world;
  if (!readWorld(worldFile, &world)) {
    printf("Error reading world.\n");
    return 1;
  }

  //printWorld(world);

  Brain brains[Color_Count];
  for (int colorInt = Color_First; colorInt < Color_Count; colorInt++) {
    Color color = Color(colorInt);

    if (!readBrain(brainFiles[color], &brains[color])) {
      printf("Error reading brain %d.\n", color);
      return 1;
    }

    string str;
    if (!formatBrain(brains[color], &str)) {
      printf("Error formatting brain %d.\n", color);
      return 1;
    }

    //printf("%s", str.c_str());
    //break;
  }

  world.setSeed(seed);

  printf("random seed: %d\n", seed);

  int i;
  for (i = 0; i <= 10000; i++) {
    printf("\n");
    printf("After round %d...\n", i);
    dumpWorld(world);

    step(&world, brains);
  }

  return 0;
}
