#include <stdio.h>
#include "World.hpp"
#include "Color.hpp"
#include "Brain.hpp"
#include "Sim.hpp"

int main(int argc, char *argv[])
{
  if (argc != 2) {
    printf("Usage: mirror <worldfile>\n");
    printf("Only guaranteed to work for even-width worlds.\n");
    return 1;
  }

  string worldFile(argv[1]);

  World world;
  if (!readWorld(worldFile, &world)) {
    printf("Error reading world.\n");
    return 1;
  }

  int half = world.getWidth() / 2;
  for (int y = 0; y < world.getHeight(); y++) {
    for (int x = 0; x < world.getWidth(); x++) {
      Cell *pCell = &world.cell(x, y);
      Cell *pFromCell = 0;

      if (y % 2 == 0) {
        if (x >= half) {
          pFromCell = &world.cell(world.getWidth() - 1 - x, y);
        }
      } else {
        if (x >= half && x != world.getWidth() - 1) {
          pFromCell = &world.cell(world.getWidth() - 2 - x, y);
        }
      }

      if (pFromCell) {
        CellType newCellType = pFromCell->getCellType();
        if (newCellType == CellType_RedAnthill) {
          newCellType = CellType_BlackAnthill;
        } else if (newCellType == CellType_BlackAnthill) {
          newCellType = CellType_RedAnthill;
        }
        pCell->setCellType(newCellType);
        pCell->setFood(pFromCell->getFood());
      }
    }
  }

  printWorldInOfficialFormat(world);

  return 0;
}
