#include <stdio.h>
#include "World.hpp"
#include "Color.hpp"
#include "Brain.hpp"
#include "Sim.hpp"

namespace
{
  bool parseInt(const string &token,
                int *pInt)
  {
    int sz = token.size();
    int result = 0;
    for (int i = 0; i < sz; i++) {
      char ch = token[i];
      if (ch < '0' || ch > '9') {
        return false;
      }
      result *= 10;
      result += (ch - '0');
    }
    *pInt = result;
    return true;
  }
}

char *worldFiles[] = {
  "../worlds/sample0.world",
  "../worlds/sample1.world",
  "../worlds/sample2.world",
  "../worlds/sample3.world",
  "../worlds/sample4.world",
  "../worlds/sample5.world",
  "../worlds/sample6.world",
  "../worlds/sample7.world",
  "../worlds/sample8.world",
  "../worlds/sample9.world"
};
const int numWorlds = sizeof(worldFiles)/sizeof(worldFiles[0]);

int main(int argc, char *argv[])
{
  if (argc < 4) {
    printf("Usage: tourney <seed> <brainfile0> <brainfile1> [<brainfile2> ...]\n");
    return 1;
  }

  int seed = 12345;
  if (!parseInt(argv[1], &seed)) {
    printf("Error parsing seed.\n");
    return 1;
  }

  World worlds[numWorlds];
  int iWorld;
  for (iWorld = 0; iWorld < numWorlds; iWorld++) {
    string worldFile = worldFiles[iWorld];
    if (!readWorld(worldFile, &worlds[iWorld])) {
      printf("Error reading world %d.\n", iWorld);
      return 1;
    }
    //worlds[iWorld].setSeed(seed);
  }

  int numBrains = argc - 2;

  vector<Brain> brains;
  brains.resize(numBrains);
  for (int iBrain = 0; iBrain < numBrains; iBrain++) {
    string brainFile = argv[iBrain + 2];
    if (!readBrain(brainFile, &brains[iBrain])) {
      printf("Error reading brain %d.\n", iBrain);
      return 1;
    }
  }

  vector<int> scores;
  scores.resize(numBrains);

  for (iWorld = 0; iWorld < numWorlds; iWorld++) {
    for (int firstBrain = 0; firstBrain < numBrains - 1; firstBrain++) {
      for (int secondBrain = firstBrain + 1;
           secondBrain < numBrains;
           secondBrain++) {
        for (int order = 0; order < 2; order++) {
          int brain0 = (order == 0 ? firstBrain : secondBrain);
          int brain1 = (order == 0 ? secondBrain : firstBrain);
          printf("World %d, red %d, black %d:  ",
                 iWorld, brain0, brain1);

          World curWorld(worlds[iWorld]);
          curWorld.setSeed(seed + order);
          Brain curBrains[2];
          curBrains[0] = brains[brain0];
          curBrains[1] = brains[brain1];

          for (int i = 0; i < 100000; i++) {
            step(&curWorld, curBrains);
          }

          int score0 = curWorld.getScore(Color_Red);
          int score1 = curWorld.getScore(Color_Black);

          printf("%3d - %3d   ", score0, score1);

          if (score0 > score1) {
            scores[brain0] += 2;
          } else if (score1 > score0) {
            scores[brain1] += 2;
          } else {
            scores[brain0] += 1;
            scores[brain1] += 1;
          }

          printf("Scores: ");
          for (int iBrain = 0; iBrain < numBrains; iBrain++) {
            if (iBrain > 0) {
              printf(", ");
            }
            printf("%d", scores[iBrain]);
          }

          printf("\n");
        }
      }
    }
  }

  return 0;
}
