#include <stdio.h>
#include "World.hpp"
#include "Color.hpp"
#include "Brain.hpp"
#include "Sim.hpp"

namespace
{
  bool parseInt(const string &token,
                int *pInt)
  {
    int sz = token.size();
    int result = 0;
    for (int i = 0; i < sz; i++) {
      char ch = token[i];
      if (ch < '0' || ch > '9') {
        return false;
      }
      result *= 10;
      result += (ch - '0');
    }
    *pInt = result;
    return true;
  }
}

int main(int argc, char *argv[])
{
  if (argc != 6) {
    printf("Usage: vis <worldfile> <redbrainfile> <blackbrainfile> <seed> <frames>\n");
    return 1;
  }

  string worldFile(argv[1]);
  string brainFiles[2];
  brainFiles[Color_Red] = argv[2];
  brainFiles[Color_Black] = argv[3];

  int seed = 12345;
  if (!parseInt(argv[4], &seed)) {
    printf("Error parsing seed.\n");
    return 1;
  }

  int frames = 15;
  if (!parseInt(argv[5], &frames)) {
    printf("Error parsing frames.\n");
    return 1;
  }

  World world;
  if (!readWorld(worldFile, &world)) {
    printf("Error reading world.\n");
    return 1;
  }

  Brain brains[Color_Count];
  for (int colorInt = Color_First; colorInt < Color_Count; colorInt++) {
    Color color = Color(colorInt);

    if (!readBrain(brainFiles[color], &brains[color])) {
      printf("Error reading brain %d.\n", color);
      return 1;
    }

    string str;
    if (!formatBrain(brains[color], &str)) {
      printf("Error formatting brain %d.\n", color);
      return 1;
    }
  }

  world.setSeed(seed);

  printf("\x1b[2J"); // clear the screen

  int i;
  for (i = 0; i <= 1000000000; i++) {
    if (i % frames == 0) {
      printf("\x1b[H"); // jump to the top of the screen

      int alive[Color_Count];
      alive[Color_Red] = 0;
      alive[Color_Black] = 0;
      for (int iAnt = 0; iAnt < world.getNumAnts(); iAnt++) {
        const Ant &ant = world.ant(iAnt);
        if (ant.getX() != -1 && ant.getY() != -1) {
          alive[ant.getColor()]++;
        }
      }

      printf("Round %d   Red %d   Black %d      Ants alive:   Red %d   Black %d     \n",
             i,
             world.getScore(Color_Red),
             world.getScore(Color_Black),
             alive[Color_Red],
             alive[Color_Black]);
      printWorldAnsi(world);
    }

    step(&world, brains);
  }

  return 0;
}
